
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>

#include "utility.h"
#include "nodelist.h"
#include "graph.h"


/******************/
/* Default values */
/******************/




/********************/
/* Output functions */
/********************/

// print help
void usage(char *c){
	fprintf(stderr,"Usage: %s -h\n",c);
	fprintf(stderr,"  -h: print current help.\n");
	fprintf(stderr,"  -i \"inputname\": input file containing the graph (default value is stdin).\n");
	fprintf(stderr,"  -o \"outputname\": output results in a file named \"outputname\" (default value is stdout).\n");
	exit(-1);
}




/**************************************************************/
/**************************************************************/
/**************************************************************/
/***************************        ***************************/
/***************************  MAIN  ***************************/
/***************************        ***************************/
/**************************************************************/
/**************************************************************/
/**************************************************************/

int main(int argc, char **argv){


///////////////////////////////////////////////////////////
//////////   DECLARATIONS AND DEFAULT VALUES   ////////////
///////////////////////////////////////////////////////////
 
	FILE* fin=NULL;
	FILE* fout=NULL;

	int i;



///////////////////////////////////////////////////////////
////////////////   PARSE COMMAND LINE   ///////////////////
///////////////////////////////////////////////////////////


	// default values
	char name_in[100]="";
	char name_out[100]="";
	int input_file_given=0;
	int output_file_given=0;

	// user's values
  
	for (i=1; i<argc; i++) {
		if ((strcmp(argv[i],"-h")==0) || (strcmp(argv[i],"--help")==0) ) {
			usage(argv[0]);     
		}
		else if ((strcmp(argv[i],"-i")==0) || (strcmp(argv[i],"--input")==0) ) {
			if (i==argc-1)
				usage(argv[0]);
			input_file_given=1;
			strcpy(name_in,argv[++i]);
		}
		else if ((strcmp(argv[i],"-o")==0) || (strcmp(argv[i],"--output")==0) ) {
			if (i==argc-1)
				usage(argv[0]);
			output_file_given=1;
			strcpy(name_out,argv[++i]);
		}
		else usage(argv[0]);
	}
    
  
  
  
  
///////////////////////////////////////////////////////////
///////////////////   OPEN FILES   ////////////////////////
///////////////////////////////////////////////////////////



	if (input_file_given==0) {
		fin=stdin;
	}
	else {
		if ( (fin=fopen(name_in,"r"))==NULL)
			report_error("name_in -- fopen: error");
	}


	if (output_file_given==0) 
		fout=stdout;
	else {
		if ( (fout=fopen(name_out,"w"))==NULL)
			report_error("name_out -- fopen: error");
	}
	


///////////////////////////////////////////////////////////
///////////////////   TREATMENT   /////////////////////////
///////////////////////////////////////////////////////////


	graph * g = graph_from_file(fin);
	
	fprintf(stderr,"nb sommets: %d\n",g->n);
	fprintf(stderr,"nb aretes: %d\n",g->m);

	int * queue;
	int * visited;
	int * comp;
	
	if( (queue=(int*)malloc(g->n*sizeof(int))) == NULL )
      	report_error("graph_from_file: malloc() error 3");
      
      if( (visited=(int*)malloc(g->n*sizeof(int))) == NULL )
      	report_error("graph_from_file: malloc() error 3");
      for (i=0;i<g->n;i++){
      	visited[i]=0;
      }
      
      if( (comp=(int*)malloc(g->n*sizeof(int))) == NULL )
      	report_error("graph_from_file: malloc() error 3");
      
      int nb=0;
      

	cell* p;	
	int u;
	int head=0;
	visited[0]=1;
	comp[nb]=0;
	nb++;
	int tail=1;
	queue[head]=0;
	
	while (head!=tail) {
		u=queue[head];
		head++;
		
		p=g->links[u]->prem;
		while (p!=NULL) {
			if (visited[p->node]==0){
				queue[tail]=p->node;
				tail++;
				visited[p->node]=1;
				comp[nb]=p->node;
				nb++;
			}
			p=p->suiv;
		}	
	}
	
/*
	for (i=0;i<nb;i++){
		fprintf(stdout,"%d %d\n",comp[i],i);
	}
*/

	int* corres;
	if( (corres=(int*)malloc(g->n*sizeof(int))) == NULL )
      	report_error("graph_from_file: malloc() error 3");
	for (i=0;i<nb;i++){
		corres[comp[i]]=i;
	}

	
	fprintf(stdout,"%d\n",nb);
	for (i=0;i<nb;i++){
		p=g->links[comp[i]]->prem;
		while (p!=NULL) {
			if (visited[p->node]==1) {
				if (i<corres[p->node]) fprintf(stdout,"%d %d\n",i,corres[p->node]);
			}
			p=p->suiv;
		}	
	}

///////////////////////////////////////////////////////////
////////////   CLOSE FILES AND FREE MEMORY  ///////////////
///////////////////////////////////////////////////////////



	if (fin!=NULL) {
		fclose(fin);
		fprintf(stderr,"Input file closed.\n");
		fflush(stderr);
	}
	
	if (fout!=NULL) {
		fclose(fout);
		fprintf(stderr,"Output file closed.\n");
		fflush(stderr);
	}





	return 0;
}

/**************************************************************/
/**************************************************************/
/*************************            *************************/
/*************************  FIN MAIN  *************************/
/*************************            *************************/
/**************************************************************/
/**************************************************************/


